/*
 *	File:	mcu_gpio.h
 *	Date:	07.01.2011
 */

#ifndef MCU_GPIO_H_
#define MCU_GPIO_H_

typedef struct
{
	volatile uint32_t configuration_register_low;
	volatile uint32_t configuration_register_high;
	volatile uint32_t input_data_register;
	volatile uint32_t output_data_register;
	volatile uint32_t bit_set_reset_register;
	volatile uint32_t bit_reset_register;
	volatile uint32_t lock_register;
} mcu_gpio_type;


typedef struct
{
  volatile uint32_t EVCR;
  volatile uint32_t MAPR;
  volatile uint32_t EXTICR[4];
  uint32_t RESERVED0;
  volatile uint32_t MAPR2;
} mcu_afio_type;

#define MCU_AFIO			((mcu_afio_type *) 0x40010000)

#define MCU_GPIOA			((mcu_gpio_type *) 0x40010800)
#define MCU_GPIOB			((mcu_gpio_type *) 0x40010C00)
#define MCU_GPIOC			((mcu_gpio_type *) 0x40011000)
#define MCU_GPIOD			((mcu_gpio_type *) 0x40011400)
#define MCU_GPIOE			((mcu_gpio_type *) 0x40011800)

//-----------------------------------------------------------------------------
#define GPIO_PIN_SPEED_2MHZ()	(2UL)
#define GPIO_PIN_SPEED_10MHZ()	(1UL)
#define GPIO_PIN_SPEED_50MHZ()	(3UL)

//-----------------------------------------------------------------------------
#define GPIO_PIN0_CONFIGURATION(PORT, MODE) \
			{ MCU_GPIO##PORT->configuration_register_low &= ~(15UL << 0); \
			  MCU_GPIO##PORT->configuration_register_low |= ((MODE) << 0); }
#define GPIO_PIN1_CONFIGURATION(PORT, MODE) \
			{ MCU_GPIO##PORT->configuration_register_low &= ~(15UL << 4); \
			  MCU_GPIO##PORT->configuration_register_low |= ((MODE) << 4); }
#define GPIO_PIN2_CONFIGURATION(PORT, MODE) \
			{ MCU_GPIO##PORT->configuration_register_low &= ~(15UL << 8); \
			  MCU_GPIO##PORT->configuration_register_low |= ((MODE) << 8); }
#define GPIO_PIN3_CONFIGURATION(PORT, MODE) \
			{ MCU_GPIO##PORT->configuration_register_low &= ~(15UL << 12); \
			  MCU_GPIO##PORT->configuration_register_low |= ((MODE) << 12); }
#define GPIO_PIN4_CONFIGURATION(PORT, MODE) \
			{ MCU_GPIO##PORT->configuration_register_low &= ~(15UL << 16); \
			  MCU_GPIO##PORT->configuration_register_low |= ((MODE) << 16); }
#define GPIO_PIN5_CONFIGURATION(PORT, MODE) \
			{ MCU_GPIO##PORT->configuration_register_low &= ~(15UL << 20); \
			  MCU_GPIO##PORT->configuration_register_low |= ((MODE) << 20); }
#define GPIO_PIN6_CONFIGURATION(PORT, MODE) \
			{ MCU_GPIO##PORT->configuration_register_low &= ~(15UL << 24); \
			  MCU_GPIO##PORT->configuration_register_low |= ((MODE) << 24); }
#define GPIO_PIN7_CONFIGURATION(PORT, MODE) \
			{ MCU_GPIO##PORT->configuration_register_low &= ~(15UL << 28); \
			  MCU_GPIO##PORT->configuration_register_low |= ((MODE) << 28); }
#define GPIO_PIN8_CONFIGURATION(PORT, MODE) \
			{ MCU_GPIO##PORT->configuration_register_high &= ~(15UL << 0); \
			  MCU_GPIO##PORT->configuration_register_high |= ((MODE) << 0); }
#define GPIO_PIN9_CONFIGURATION(PORT, MODE) \
			{ MCU_GPIO##PORT->configuration_register_high &= ~(15UL << 4); \
			  MCU_GPIO##PORT->configuration_register_high |= ((MODE) << 4); }
#define GPIO_PIN10_CONFIGURATION(PORT, MODE) \
			{ MCU_GPIO##PORT->configuration_register_high &= ~(15UL << 8); \
			  MCU_GPIO##PORT->configuration_register_high |= ((MODE) << 8); }
#define GPIO_PIN11_CONFIGURATION(PORT, MODE) \
			{ MCU_GPIO##PORT->configuration_register_high &= ~(15UL << 12); \
			  MCU_GPIO##PORT->configuration_register_high |= ((MODE) << 12); }
#define GPIO_PIN12_CONFIGURATION(PORT, MODE) \
			{ MCU_GPIO##PORT->configuration_register_high &= ~(15UL << 16); \
			  MCU_GPIO##PORT->configuration_register_high |= ((MODE) << 16); }
#define GPIO_PIN13_CONFIGURATION(PORT, MODE) \
			{ MCU_GPIO##PORT->configuration_register_high &= ~(15UL << 20); \
			  MCU_GPIO##PORT->configuration_register_high |= ((MODE) << 20); }
#define GPIO_PIN14_CONFIGURATION(PORT, MODE) \
			{ MCU_GPIO##PORT->configuration_register_high &= ~(15UL << 24); \
			  MCU_GPIO##PORT->configuration_register_high |= ((MODE) << 24); }
#define GPIO_PIN15_CONFIGURATION(PORT, MODE) \
			{ MCU_GPIO##PORT->configuration_register_high &= ~(15UL << 28); \
			  MCU_GPIO##PORT->configuration_register_high |= ((MODE) << 28); }

//-----------------------------------------------------------------------------
#define GPIO_PIN_CONFIGURATION_ANALOG(PORT, PIN, LEVEL, MODE, SPEED) \
			{ GPIO_PIN##PIN##_CONFIGURATION(PORT, 0UL) }

#define GPIO_PIN_CONFIGURATION_INPUT_FLOATING(PORT, PIN, LEVEL, MODE, SPEED) \
			{ GPIO_PIN##PIN##_CONFIGURATION(PORT, 4UL) }

#define GPIO_PIN_CONFIGURATION_INPUT_PULL_DOWN(PORT, PIN, LEVEL, MODE, SPEED) \
			{ GPIO_PIN##PIN##_CONFIGURATION(PORT, 8UL); \
			  MCU_GPIO##PORT->bit_set_reset_register |= (1UL << PIN) << 16; }

#define GPIO_PIN_CONFIGURATION_INPUT_PULL_UP(PORT, PIN, LEVEL, MODE, SPEED) \
			{ GPIO_PIN##PIN##_CONFIGURATION(PORT, 8UL)  \
	          MCU_GPIO##PORT->bit_set_reset_register |= (1UL << PIN); }

#define GPIO_PIN_CONFIGURATION_GENERAL_OUTPUT_PUSH_PULL(PORT, PIN, LEVEL, MODE, SPEED) \
			{ GPIO_PIN##PIN##_CONFIGURATION(PORT, 0UL | GPIO_PIN_##SPEED()) }

#define GPIO_PIN_CONFIGURATION_GENERAL_OUTPUT_OPEN_DRAIN(PORT, PIN, LEVEL, MODE, SPEED) \
			{ GPIO_PIN##PIN##_CONFIGURATION(PORT, 4UL | GPIO_PIN_##SPEED()) }

#define GPIO_PIN_CONFIGURATION_ALTERNATE_OUTPUT_PUSH_PULL(PORT, PIN, LEVEL, MODE, SPEED) \
			{ GPIO_PIN##PIN##_CONFIGURATION(PORT, 8UL | GPIO_PIN_##SPEED()) }

#define GPIO_PIN_CONFIGURATION_ALTERNATE_OUTPUT_OPEN_DRAIN(PORT, PIN, LEVEL, MODE, SPEED) \
			{ GPIO_PIN##PIN##_CONFIGURATION(PORT, 12UL | GPIO_PIN_##SPEED()) }

//-----------------------------------------------------------------------------
#define GPIO_PIN_CONFIGURATION(PORT, PIN, LEVEL, MODE, SPEED) \
			{ \
			  if((MCU_RCC->APB2_peripheral_clock_enable_register & MCU_RCC_GPIO##PORT) != MCU_RCC_GPIO##PORT) \
				{ \
			  	  MCU_RCC->APB2_peripheral_clock_enable_register |= MCU_RCC_GPIO##PORT; \
				} \
			  GPIO_PIN_CONFIGURATION_##MODE(PORT, PIN, LEVEL, MODE, SPEED); \
			}

//-----------------------------------------------------------------------------
#define GPIO_PIN_ON_HIGH(PORT, PIN, LEVEL, MODE, SPEED) \
			{ MCU_GPIO##PORT->bit_set_reset_register |= (1UL << PIN); }

#define GPIO_PIN_ON_LOW(PORT, PIN, LEVEL, MODE, SPEED) \
			{ MCU_GPIO##PORT->bit_set_reset_register |= (1UL << PIN) << 16; }

#define GPIO_PIN_OFF_HIGH(PORT, PIN, LEVEL, MODE, SPEED) \
			{ MCU_GPIO##PORT->bit_set_reset_register |= (1UL << PIN) << 16; }

#define GPIO_PIN_OFF_LOW(PORT, PIN, LEVEL, MODE, SPEED) \
			{ MCU_GPIO##PORT->bit_set_reset_register |= (1UL << PIN); }

//-----------------------------------------------------------------------------
#define GPIO_PIN_ON(PORT, PIN, LEVEL, MODE, SPEED) \
			{ GPIO_PIN_ON_##LEVEL(PORT, PIN, LEVEL, MODE, SPEED) }

#define GPIO_PIN_OFF(PORT, PIN, LEVEL, MODE, SPEED) \
			{ GPIO_PIN_OFF_##LEVEL(PORT, PIN, LEVEL, MODE, SPEED) }

//-----------------------------------------------------------------------------
#define GPIO_PIN_SIGNAL_HIGH(PORT, PIN, LEVEL, MODE, SPEED) \
			( (MCU_GPIO##PORT->input_data_register & (1UL << PIN)) == (1UL << PIN) )

#define GPIO_PIN_SIGNAL_LOW(PORT, PIN, LEVEL, MODE, SPEED) \
		( (MCU_GPIO##PORT->input_data_register & (1UL << PIN)) != (1UL << PIN) )

#define GPIO_PIN_SIGNAL(PORT, PIN, LEVEL, MODE, SPEED) \
			( GPIO_PIN_SIGNAL_##LEVEL(PORT, PIN, LEVEL, MODE, SPEED) )

//-----------------------------------------------------------------------------
#define PIN_CONFIGURATION(PIN_DESCRIPTION) GPIO_PIN_CONFIGURATION(PIN_DESCRIPTION)
#define PIN_ON(PIN_DESCRIPTION) GPIO_PIN_ON(PIN_DESCRIPTION)
#define PIN_OFF(PIN_DESCRIPTION) GPIO_PIN_OFF(PIN_DESCRIPTION)
#define PIN_SIGNAL(PIN_DESCRIPTION) GPIO_PIN_SIGNAL(PIN_DESCRIPTION)

#endif /* MCU_GPIO_H_ */
